#!/usr/bin/env python
# -*- coding: utf-8 -*-

###########################################################################
#    Copyright (C) 2008 by Ing. Tomáš Košan                                      
#    <t.kosan@k25.cz>                                                             
#
# Copyright: See COPYING file that comes with this distribution
#
###########################################################################


from PyQt4.QtGui import * 
from PyQt4.QtCore import *

from ut2XXX import utils


# main scene class with prerendered grid

class DSO_Scene(QGraphicsScene):
	
	def __init__(self):
		QGraphicsScene.__init__(self, 0,0,540,440)
		
		# we want black background
		bg = QBrush()
		bg.setColor(QColor(Qt.black))
		bg.setStyle(Qt.SolidPattern)
		self.setBackgroundBrush(bg)
		
		# next paint grid
		self.grid = DSO_grid(self)
		
		# create 3 cursors
		self.ch1_cursor = DSO_cursor(self, "1", Qt.cyan, "Y")
		self.ch2_cursor = DSO_cursor(self, "2", Qt.yellow, "Y")
		self.chX_cursor = DSO_cursor(self, "", Qt.red, "X")
		
		self.CH1_info = DSO_channel(self, "Test<br>Test2<br>Test3")
		
		self.wave1 = DSO_wave(self, Qt.cyan)
		self.wave1.hide()
		
		self.wave2 = DSO_wave(self, Qt.yellow)
		self.wave2.hide()
		
		# finally update screen
		self.update()
		
		self.pixmap = QGraphicsPixmapItem(None, self)
		
	def updateScreen(self, ch1_data, ch2_data):
		self.grid.show()
		if ch1_data["active"]:
			self.wave1.repaint(ch1_data["samples"])
			self.ch1_cursor.setP(2*(106-ch1_data["y_offset"]))
			self.ch1_cursor.show()
		else:
			self.wave1.hide()
			self.ch1_cursor.hide()
				
		if ch2_data["active"]:	
			self.wave2.repaint(ch2_data["samples"])	
			self.ch2_cursor.setP(2*(106-ch2_data["y_offset"]))
			self.ch2_cursor.show()
		else:
			self.wave2.hide()
			self.ch2_cursor.hide()
				
		if ch2_data["y_offset"] in range(3,250):		
			self.chX_cursor.setP(2*(ch2_data["x_offset"])+14)
		elif ch2_data["y_offset"] in range(0,3):
			self.chX_cursor.setP(20)
		elif ch2_data["y_offset"] > 250:
			self.chX_cursor.setP(270)
			
		self.pixmap.hide()
		self.update()	
			
	def showPixmap(self, pixmap):
		
		self.wave1.hide()
		self.ch1_cursor.hide()
		self.wave2.hide()
		self.ch2_cursor.hide()
		self.grid.hide()
		
		print "Showing pixmap"
		#self.pixmap = QGraphicsPixmapItem(pixmap, None, self)
		self.pixmap.setPixmap(pixmap)
		self.pixmap.setPos(-50,-20)
		self.pixmap.show()
		self.update()
		
		
# wave is 
class DSO_wave(QGraphicsItemGroup):
		
	def __init__(self, parent, color):
		QGraphicsItemGroup.__init__(self, None, parent)
		
		self.color = QColor(color)
		self.pen = QPen()
		self.pen.setColor(self.color)
		
		self.lines = []
		
		self.offset_x = 10
		
		for i in range(self.offset_x,250+self.offset_x):
			line = QGraphicsLineItem(self)
			line.setLine(i*2,20+200,(i*2)+2,20+200)
			line.setPen(self.pen)
			self.lines.append(line)
			
			
	# hide all		
	def hide(self):
		for line in self.lines:
			line.hide()
				
	def repaint(self, samples):
		index = 0
		x = 20
		y = 20
		for sample in samples:
			line = self.lines[index]
			index += 1 
			# first point
			if x == 20:
				line.setLine(x,(2*(255 - sample) - 34),x,(2*(255 - sample) - 34))
			else:
				line.setLine(x-2,y,x,(2*(255 - sample) - 34))
				
			x = x + 2
			y = (2*(255 - sample) - 34)
			#line.setPen(self.pen)
			line.show()


class DSO_channel(QGraphicsPolygonItem):

	def __init__(self, parent, txt="", color=Qt.white):
		QGraphicsPolygonItem.__init__(self, None, parent)
		
		polygon = QPolygonF()
	
		polygon.append(QPointF(0,0))
		polygon.append(QPointF(0,16))
		polygon.append(QPointF(12,16))
		polygon.append(QPointF(20,8))
		polygon.append(QPointF(12,0))

		bg = QBrush()
		col = QColor(color)
		col.setAlpha(200)
		bg.setColor(col)
		bg.setStyle(Qt.SolidPattern)
		self.setBrush(bg)
		text = QGraphicsTextItem(self, parent)
		text.setHtml(txt)
		self.show()
		

# 
class DSO_cursor(QGraphicsPolygonItem):
		
	def __init__(self, parent, name="", color=Qt.white, typ="X"):
		QGraphicsPolygonItem.__init__(self, None, parent)
		
		polygon = QPolygonF()
		
		if typ == "Y":
			polygon.append(QPointF(0,0))
			polygon.append(QPointF(0,16))
			polygon.append(QPointF(12,16))
			polygon.append(QPointF(20,8))
			polygon.append(QPointF(12,0))
		
		if typ == "X":
			polygon.append(QPointF(0,0))
			polygon.append(QPointF(6,6))
			polygon.append(QPointF(12,0))
			
		
		self.setPolygon(polygon)
			
		# we want black background
		bg = QBrush()
		col = QColor(color)
		col.setAlpha(200)
		bg.setColor(col)
		bg.setStyle(Qt.SolidPattern)
		self.setBrush(bg)
		
		if typ == "Y":
			text = QGraphicsTextItem(self, parent)
			text.setPlainText(name)
		
		self.typ = typ
		self.show()

	def setP(self, pos):
		if self.typ == "Y":
			self.setPos(0,pos)
		if self.typ == "X":
			self.setPos(pos,10)
	
# paint basic grid	
	 
class DSO_grid(QGraphicsItemGroup):
	
	def __init__(self, parent):
		QGraphicsItemGroup.__init__(self, None, parent)
		self.showGrid()
		#self.hide()

	def showGrid(self):
		self.color = QColor()
		pen = QPen()
		self.color.setNamedColor("white")
		pen.setColor(self.color)
		
		# we will generate DSO screen grid
		for x in range(20,521,10):
			for y in range(20,421,50):
				
				self.point = QGraphicsLineItem(self)
				
				if y == 20:
					self.point.setLine(x,y,x,y+3)
				elif y == 220:
					self.point.setLine(x,y-2,x,y+2)
				elif y == 420:
					self.point.setLine(x,y,x,y-3)
				else:
					self.point.setLine(x,y,x,y)
				
				self.point.setPen(pen)
				self.point.show()

		for x in range(20,521,50):
			for y in range(20,421,10):
				
				self.point = QGraphicsLineItem(self)
				
				if x == 20:
					self.point.setLine(x,y,x+3,y)
				elif x == 270:
					self.point.setLine(x-2,y,x+2,y)
				elif x == 520:
					self.point.setLine(x,y,x-3,y)
				else:
					self.point.setLine(x,y,x,y)
				
				self.point.setPen(pen)
				self.point.show()
		